#!/bin/ksh

#
# This script will pull the headers from the MailScanner quarantine/*/spam
# directory and send an email notifying recipients of spam that wasn't delivered
# to them.  The email will include a link that will automatically send the
# email along.
#
# Other files needed to actually send the email are:
# spamAssassinRequest.cgi - used for the "REQUEST THIS FILE" link)
# spamAssassinRequest.ksh - used via an email alias to set up the running of
#                           spamAssassinRequestCron.ksh
# spamAssassinRequestCron.ksh - used to actually send the quarantined email
#

DATE=`date '+%Y%m%d'`
TODAY=`date '+%Y%b%d'`
SPAMDIR=/var/spool/MailScanner/quarantine/$DATE/spam
SPAMFILE=spam.caught
# SPAMFILE=spam.caught.test
cd $SPAMDIR

#
# This script cron runs at 11:55 p.m., so sleep for 10 minutes, in case of more
# spam arrives before MailScanner creates the next day's quarantine directory
#
sleep 600

#
# Most likely not needed, but create the $SPAMFILE here first.
#
cat /dev/null > $SPAMFILE

#
# Go through the qf files and pull out the displayed Date, Subject, From and To
# headers
#
ls qf* | cut -c3-16 | while read emailMessageID ; do
   EMAILDATE=$(grep -i ^H?*?*Date: qf$emailMessageID | head -1 | cut -c 4-100)
   SUBJECT=$(grep -i ^H?*?*Subject: qf$emailMessageID | cut -c 4-100)
   FROM=$(grep -i ^H?*?*From: qf$emailMessageID | cut -c 4-100)
   TO=$(grep -i ^H?*?*To: qf$emailMessageID | head -1 | cut -c 4-100)
#
# Pull the actual recipients.  For each recipient, create their very own entry
# in the $SPAMFILE.  This is done, because later in this script, we look for
# lines with "HEADER INFO.*Subject"  If there is more than one actual recipient,
# the Subject header would wrap to the next line.
#
   recipient=$(grep ^R qf$emailMessageID | sed 's/R.*://g')
   print $recipient | sed 's/ /\
/g' | while read RECIPIENT ; do
      print "HEADER INFO|$DATE|$emailMessageID|$EMAILDATE|$FROM|$TO|$RECIPIENT|$SUBJECT\n\n" >> $SPAMFILE
   done
done

#
# Get rid of the <> in the $SPAMFILE, since it breaks part of this code and
# I don't feel like figuring out why, at the moment
#
sed 's/[<>]//g' $SPAMFILE > /tmp/$SPAMFILE.$$ && mv /tmp/$SPAMFILE.$$ $SPAMFILE

#
# Create a file that will give me stats of how many emails were caught for whom
#
print "$SPAMFILE for $TODAY: \c" > /tmp/spammailnotify
grep -c "HEADER INFO" $SPAMFILE >> /tmp/spammailnotify
print "\n" >> /tmp/spammailnotify

#
# Find the $email addresses to which the emails were sent.  Look for upper and
# lower case and lump them together as one address (sort -fu)
#
grep "^HEADER INFO.*Subject" $SPAMFILE | awk -F'|' '{print $7}' | sed 's/[<>]//g' | sed 's/ //g' | sort -fu | while read email ; do

#
# Create an HTML file that will eventually be mailed to the above $email
# address.  Do not indent the headers, or it won't work.
#
cat << EOF > /tmp/$email.$$
From: filtered.email@ihs.com
To: $email
Subject: Filtered email for $email from $TODAY
MIME-Version: 1.0
Content-Type: text/html; charset=us-ascii
Content-Disposition: inline

   <HTML>
   <P>The following headers are from email that was determined to be junk email
      for the following email address:

   <P><CENTER><B>$email</B></CENTER>

   <HR width=50%>
EOF

#
# Find all the "header information" lines that belong to the above $email
# address, pull the information and print it to their /tmp/$email.$$ file
#
   # $1 = "HEADER INFO" (this text not needed, but legacy from an old script)
   # $2 = $DATE (date of the MailScanner quarantined directory)
   # $3 = emailID
   # $4 = Date: header
   # $5 = From: header
   # $6 = To: header
   # $7 = actual recipient in qf file ($email)
   # $8 = Subject: header

#
# The following includes the "To:" header.  It is commented out, because it
# confused some people to see the "To:" header..."Why am I getting this
# notification, if it wasn't sent to me?"
#
   # grep -i "^HEADER INFO|.*|.*|.*|.*|.*|<*$email>*|Subject:" $SPAMFILE | awk -F'|' '{print "<P>" $5 "\n\t<BR>&nbsp;&nbsp;&nbsp;" $6 "\n\t<BR>&nbsp;&nbsp;&nbsp;Date: " $4 "\n\t<BR>&nbsp;&nbsp;&nbsp;" $8 "\n\t<BR>&nbsp;&nbsp;&nbsp;<A HREF=\"http://tsg.ihs.com/tsg-bin/spamAssassinRequest.cgi?emp_email="$7"&emailID="$3"&spamdate="$2"\">REQUEST THIS EMAIL</A> &nbsp;&nbsp;&nbsp; (<A HREF=\"mailto:spam.request@ihs.com?subject="$3":"$7":"$2"&body=Send%20this%20email,%20and%20you%20should%20receive%20your%20requested%20email.\">I am not connected to the IHS intranet</a>)<P><HR width=50%>"}' >> /tmp/$email.$$

   grep -i "^HEADER INFO|.*|.*|.*|.*|.*|<*$email>*|Subject:" $SPAMFILE | awk -F'|' '{print "<P>" $5 "\n\t<BR>&nbsp;&nbsp;&nbsp;Date: " $4 "\n\t<BR>&nbsp;&nbsp;&nbsp;" $8 "\n\t<BR>&nbsp;&nbsp;&nbsp;<A HREF=\"http://tsg.ihs.com/tsg-bin/spamAssassinRequest.cgi?emp_email="$7"&emailID="$3"&spamdate="$2"\">REQUEST THIS EMAIL</A> &nbsp;&nbsp;&nbsp; (<A HREF=\"mailto:spam.request@ihs.com?subject="$3":"$7":"$2"&body=Send%20this%20email,%20and%20you%20should%20receive%20your%20requested%20email.\">I am not connected to the IHS intranet</a>)<P><HR width=50%>"}' >> /tmp/$email.$$

#
# After the above is done, finish creating the /tmp/$email.$$ file
#
cat << EOF >> /tmp/$email.$$
   <P>Email is determined to be junk email due to certain words, or phrases
      found in the body of the email message.

   <P>If you feel this was a valid, <B>IHS business-related</B> email, you may
      request a copy of the email by clicking on the
      <FONT COLOR="blue"><U>REQUEST THIS EMAIL</U></FONT> link in the email
      group.

   <P><B>NOTE:</B> If you are not connected to the IHS intranet, i.e. not
      connected directly on campus, or not using RAS, click on the link that
      says, &quot;<FONT COLOR="blue"><U>I am not connected to the IHS
      intranet</U></FONT>&quot; and follow the instructions.

   <P>Suggestions to reduce the amount of junk email you receive:

   <P>
   <UL>
      <LI>Never use your ihs.com email address when subscribing to mailing
          lists, buying products on-line, posting on message boards, entering
          contests, sweepstakes, etc.
      <LI>Never reply to junk email, even if it states you will be removed from
          their list.  Replying only validates the existence of your email
          address and most likely will get your address added to more lists.
          This includes clicking on any sort of "unsubscribe" link.
      <LI>Never buy something from someone who sends you unsolicited
          advertising.
   </UL>

   <P>Not all junk email can be stopped at the company email gateway.  If you
      take steps to keep your email address private, there will be less of a
      chance you will receive any at all.

   <P>For more information, please visit
      <A HREF="http://tsg.ihs.com/email.htm">http://tsg.ihs.com/email.htm</A>
   </HTML>
EOF

#
# Email the /tmp/$email.$$ file to the $email address
#
   /usr/bin/mail $email < /tmp/$email.$$
   # /usr/bin/mail dustin.baer@ihs.com < /tmp/$email.$$

   rm /tmp/$email.$$

#
# Print how many emails were caught for this person to the stats file
# (/tmp/spammailnotify) file that gets mailed to me
#
   print "$email - \c" >> /tmp/spammailnotify

   grep -ci "^HEADER INFO|.*|.*|.*|.*|.*|<*$email>*|Subject:" $SPAMFILE >> /tmp/spammailnotify

done

#
# Mail the stats file to me
#

mailx -s "Filtered notifications for $TODAY" dustin.baer@ihs.com < /tmp/spammailnotify

rm /tmp/spammailnotify
