#!/bin/ksh

#
# This script will check for a file created by spamAssassinRequest.ksh and
# send the original email caught by SpamAssassin to the person who requested, or
# create a new email if there are more than one actual recipients.  It runs via
# a root cronjob every 15 minutes.
#
# If the email no longer exists, then an error message will be sent.
#

DIR=/var/spool/MailScanner

if [[ -s $DIR/spamAssassin.request ]] ; then

#
# Some people get click happy, or impatient and send multiple requests.
#
   sort -u $DIR/spamAssassin.request > $DIR/spamAssassin.request.$$ && rm $DIR/spamAssassin.request

#
# Get the three pieces of information in the request
#
   awk -F'|' '{print $1, $2, $3}' $DIR/spamAssassin.request.$$ | while read emailID emp_email spamdate; do
   
#
# Check to see that the quarantined spam email still exists
#

      if [[ -f $DIR/quarantine/$spamdate/spam/qf$emailID ]] ; then
#
# Add a header, so that when it goes back through MailScanner, it won't be
# tagged as spam, but will still be checked for viruses.
#

sed 's/^.$/H??X-SpamRequested-Email: Requested\
./' $DIR/quarantine/$spamdate/spam/qf$emailID > $DIR/quarantine/$spamdate/spam/qf$emailID.$$ && mv $DIR/quarantine/$spamdate/spam/qf$emailID.$$ $DIR/quarantine/$spamdate/spam/qf$emailID

#
# There is a possibility that there are multiple recipients, so better check for
# multiple recipient headers in the qf file.
#

         let multipleRecipients=$(grep "^[rR]" $DIR/quarantine/$spamdate/spam/qf$emailID | grep -v "$emp_email" | wc -l | awk '{print $1}')

         if (( $multipleRecipients == 0 )) ; then

#
# There is only one recipient, so just copy the qf and df files to
# /var/spool/mqueue.in for virus scanning.
#

            cp $DIR/quarantine/$spamdate/spam/df$emailID /var/spool/mqueue.in
            cp $DIR/quarantine/$spamdate/spam/qf$emailID /var/spool/mqueue.in
         else

#
# There are multiple recipients, so we need to build a new qf file
# without all of the other recipients.  We do this by grep -n to
# find where the recipients begin, `head` everything before that to
# a file, grep the actual recipient from the qf file, then grep -n
# again to find the end of the recipients and `tail` everything
# after that to the new qf file.
#
# When the new qf file is built, cp the original df file and the
# new qf file to /var/spool/mqueue.in.
#

            let endBeginHeaderLine=$(grep -n ^S $DIR/quarantine/$spamdate/spam/qf$emailID | cut -d: -f1)
            head -$endBeginHeaderLine $DIR/quarantine/$spamdate/spam/qf$emailID > /tmp/qf$emailID.$emp_email
   
            grep "^[rR].*$emp_email" $DIR/quarantine/$spamdate/spam/qf$emailID >> /tmp/qf$emailID.$emp_email
   
            let beginEndHeaderLine=$(grep -ni "return-path.*g>$" $DIR/quarantine/$spamdate/spam/qf$emailID | cut -d: -f1)
            let totalHeaderLines=$(wc -l $DIR/quarantine/$spamdate/spam/qf$emailID | awk '{print $1}')
            let tailNumber=$totalHeaderLines-$beginEndHeaderLine+1
            tail -$tailNumber $DIR/quarantine/$spamdate/spam/qf$emailID  >> /tmp/qf$emailID.$emp_email
            cp $DIR/quarantine/$spamdate/spam/df$emailID /var/spool/mqueue.in
            mv /tmp/qf$emailID.$emp_email /var/spool/mqueue.in/qf$emailID
         fi
      else
   
#
# If the email message no longer exists, send an error message.
#

         print "From: filtered.email@ihs.com" >> /tmp/$emailID.$emp_email
         print "Subject: Cannot find $emailID for $emp_email" >> /tmp/$emailID.$emp_email
         print "I cannot locate the following email:\n" >> /tmp/$emailID.$emp_email
         print "Requested email date: $spamdate" >> /tmp/$emailID.$emp_email
         print "Requested email id: $emailID" >> /tmp/$emailID.$emp_email
         print "Recipient address: $emp_email" >> /tmp/$emailID.$emp_email
         print "\nMost likely this email truly is spam and removed from the server to save space." >> /tmp/$emailID.$emp_email
         print "\nIf you think you have done everything correctly, please forward this email \c" >> /tmp/$emailID.$emp_email
         print "to filtered.email@ihs.com and ask for assistance." >> /tmp/$emailID.$emp_email
         /usr/lib/sendmail $emp_email < "/tmp/$emailID.$emp_email"
         rm "/tmp/$emailID.$emp_email"
      fi
   done

   rm $DIR/spamAssassin.request.$$
fi
